//======================================================= file = common.h =====
//=  Include file for VOQ switch simulation model                             =
//=============================================================================
//=  Notes:  1) This file contains                                            =
//=               - Constants                                                 =
//=               - Common (external) variables                               =
//=               - Common (external) CSIM types                              =
//=               - Function prototypes                                       =
//=          2)  Basic time unit is the millisecond                           =
//=---------------------------------------------------------------------------=
//=  History:  KJC (01/26/02) - Genesis                                       =
//=============================================================================
#pragma warn -pro                    // Disable function prototype warning
#pragma warn -stu                    // Disable undefined structure warning
#pragma warn -rch                    // Disable unreachable code warning
#pragma warn -ccc                    // Disable condition always true warning

//----- Conditional definitions -----------------------------------------------
#if defined(MAIN)                    // Determine if main or external
  #define EXTERN
#endif
#if defined(EXTERNAL)
  #define EXTERN extern
#endif

//----- Constants -------------------------------------------------------------
#define VERSION "3.00 <01/26/02>"    // Version number
#define OUT_FILE     "result.txt"    // Name of output file

#define FALSE                  0     // FALSE = 0
#define TRUE                   1     // TRUE = 1

#define MEDIA_RATE        10.0e6     // Media rate in bits per millisec
#define STATUS_TIME           64     // Status update time in bytes
#define SPEED_UP             1.0     // Internal transfer speed-up
#define CP_THRESH           1500     // CP threshold for transfer enabled
#define XFER_TYPE              1     // 1 = block dist, 2 = cell dist
#define BLOCK_SIZE          1500     // Block size for transfer
#define CELL_SIZE            100     // Cell size for transfer
#define N                      2     // Number of input ports in the switch
#define GEN_FLAG               1     // 1 = IEEE dist, 2 = USF dist

//----- External variables ----------------------------------------------------
EXTERN double    Sim_time;           // Simuation time
EXTERN double    Lambda1;            // Arrival rate for generate1 in pkts/ms
EXTERN double    Lambda2;            // Arrival rate for generate2 in pkts/ms
EXTERN int       VOQ_buf_len[N][N];  // Level of VOQ[i][j] in bytes
EXTERN int       CP_buf_len[N][N];   // Level of CP[i][j] in bytes
EXTERN int       HOL_frag_len[N][N]; // VOQ HOL fragment length

//----- CSIM types ------------------------------------------------------------
EXTERN FACILITY Out_link[N];         // Facility for output port link
EXTERN EVENT    VOQ_first[N][N];     // Input port VOQ HOL for packets
EXTERN EVENT    VOQ_event[N][N];     // Input port VOQ buffers for packets
EXTERN EVENT    CP_event[N][N];      // Crossbar crosspoint queues for packets
EXTERN EVENT    Wait_xfer[N];        // Wait for a transfer event
EXTERN EVENT    State_change[N];     // State change in an in_id buffer
EXTERN EVENT    Xfer_voq[N];         // VOQ-to-CP buffer xfer done for in_id
EXTERN EVENT    Xfer_cp[N];          // VOQ-to-CP buffer xfer done for out_id
EXTERN EVENT    Xmit_done[N];        // CP-to-link xmit done event for out_id
EXTERN TABLE    Delay_table[N];      // Delay table for packet delay
EXTERN TABLE    Bytes_table[N];      // Bytes transfered table

//----- Function prototypes ---------------------------------------------------
// Traffic generators
EXTERN void generate1(int in_id);
EXTERN void generate2(int in_id);
EXTERN void (* gen[N])(int in_id);

// Input ports with VOQ (VOQ arbiter in each input port)
EXTERN void voq_arbit_b(int in_id);
EXTERN void voq_arbit_c(int in_id);
EXTERN void (* voq_arb[N])(int in_id);
EXTERN void status_reporter(int in_id);
EXTERN void (* stat[N])(int in_id);
EXTERN void inport(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void (* in[N])(int in_id, int out_id, int pkt_len, double org_time);

// Output ports (CP arbiter in each output port)
EXTERN void outport(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void (* out[N])(int in_id, int out_id, int pkt_len, double org_time);
EXTERN void cp_arbit(int out_id);
EXTERN void (* cp_arb[N])(int out_id);

// Run-time simulation status reporter
EXTERN void run_sim(double run_time, int run_num);
