//====================================================== file = outport.c =====
//=  Output port for VOQ switch simulation model                              =
//=============================================================================
//=  Notes: 1)                                                                =
//=---------------------------------------------------------------------------=
//=  Contains: cp_arbit()                                                     =
//=            outport()                                                      =
//=---------------------------------------------------------------------------=
//=  History:  KJC (01/26/02) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include <assert.h>     // Needed for assert() macro
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//=============================================================================
//==  Poller for crosspoint buffers                                          ==
//=============================================================================
void cp_arbit(int out_id)
{
  int      in_id;               // Input port id
  int      in_poll;             // Input port id to be polled
  int      flag;                // A flag variable
  int      i;                   // Loop counter

  create("cp_arbit");

  // Poll output port CP buffers forever
  in_id = 0;
  while(1)
  {
    // Test if any CP buffers are non-empty
    flag = FALSE;
    for (i=0; i<N; i++)
      if (queue_cnt(CP_event[i][out_id]) > 0) flag = TRUE;

    // If all CP buffers are empty, wait for a transfer into a CP buffer
    if (flag == FALSE) wait(Xfer_cp[out_id]);

    // Poll one "full circle"
    for (i=0; i<N; i++)
    {
      in_poll = (in_id + i) % N;

      if ((queue_cnt(CP_event[in_poll][out_id]) > 0) &&
          (status(Out_link[out_id]) == FREE))
      {
        set(CP_event[in_poll][out_id]);
        wait(Xmit_done[out_id]);
        hold(0.01 / MEDIA_RATE);         // Tiny wait for deadlock break
        in_id = (in_poll + 1) % N;
        break;
      }
    }
  }
}

//=============================================================================
//==  Output port "owning" the CP buffers                                    ==
//=============================================================================
void outport(int in_id, int out_id, int pkt_len, double org_time)
{
  int      i;                   // Loop counter

  create("outport");

  // Queue the transferred packet in the appropriate crosspoint buffer
  assert(state(CP_event[in_id][out_id]) == NOT_OCC);
  CP_buf_len[in_id][out_id] = CP_buf_len[in_id][out_id] + pkt_len;
  queue(CP_event[in_id][out_id]);

  // Set the state change for allowing the next packet to be transferred in
  set(State_change[in_id]);

  // Transmit packet, record delay and byte count, and set appropriate events
  assert(status(Out_link[out_id]) == FREE);
  reserve(Out_link[out_id]);
  for (i=0; i<pkt_len; i++)
  {
    hold(8.0 / MEDIA_RATE);
    CP_buf_len[in_id][out_id]--;
  }
  assert(CP_buf_len[in_id][out_id] >= 0);
  set(Xmit_done[out_id]);
  record((clock - org_time), Delay_table[out_id]);
  record(pkt_len, Bytes_table[out_id]);
  release(Out_link[out_id]);

  return;
}
